﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using TiledSharp;

namespace tmxParser
{
    public class Program
    {      
        static void Main(string[] args)
        { 

            //our tilemap
            byte[] tileMap = new byte[64 * 64];
            byte[] objectsMap = new byte[64 * 64];
            byte[] triggersMap = new byte[64 * 64+256];

            byte[] shadowMap = new byte[256];
            byte[] priorities = new byte[256];
            byte[] collisions = new byte[256];
            byte[] randoms = new byte[256];
            byte[] triggers = new byte[256];
            //byte[] randoms3 = new byte[256];

            Random rand = new Random(333);

            //Generate random numbers
            for (int i=0;i<256;i++)
            {
                randoms[i] = (byte)rand.Next(0, 4);
                // triggers[i] = 255;//fill with 255
              //  randoms3[i] = (byte)rand.Next(0, 3);
            }


            // File.WriteAllBytes("random10.bin", randoms);
            File.WriteAllBytes("randomChest.bin", randoms);



            List<GameObject> gameObjects = new List<GameObject>();



            Console.WriteLine("TMX Parser by Denis Grachev");
            Console.WriteLine("===========================");
            
            TmxMap map = new TmxMap(args[0]);


            int totalTiles = map.Tilesets[0].Tiles.Count();


            //////////////////////////////////////////////////////////
            Console.WriteLine("Parsing Tiles Priorities and Collisions and Shadows!");
            //this property in tileset            
            int tilesInTileset = map.Tilesets[0].Tiles.Count();

            for (int i = 0; i < 256; i++)
            {
                try
                {
                    double prob = map.Tilesets[0].Tiles[i].Probability;
                    if (prob == 0)
                        priorities[i] = 1;

                    //try get shadowmap
                    try
                    {
                        int shadowNum = Int32.Parse(map.Tilesets[0].Tiles[i].Properties["shadow"]);
                        shadowMap[i] = (byte)shadowNum;

                    }
                    catch
                    {

                    }


                    string tileType = map.Tilesets[0].Tiles[i].Type;

                    switch (tileType)
                    {
                        case "solid":
                            collisions[i] = 1;
                            break;
                        case "chest":
                            collisions[i] = 2;
                            break;

                        case "door":
                            collisions[i] = 3;
                            break;
                        case "reshetka":
                            collisions[i] = 4;
                            break;
                        case "smallDoor":
                            collisions[i] = 5;
                            break;
                        case "smallReshetka":
                            collisions[i] = 6;
                            break;

                        case "exit":
                            collisions[i] = 254;
                            break;

                        default:
                            collisions[i] = 0;
                            break;
                    }

                }
                catch
                {
                    //no such field
                }

            }


            Console.WriteLine("Parsing tilemap!");
            Console.WriteLine("TOTAL TILES: "+ map.TileLayers["map"].Tiles.Count().ToString());

            //import tiles from "tilemap" tilelayer
            for (int j = 0; j < 64; j++)
            {
                for (int i = 0; i < 64; i++)
                {
                    int tile = (map.TileLayers["map"].Tiles[i + j * 64].Gid);



                    //onle top left corner of tile checks for auto collisions
                    /*
                    if (i % 2 == 0 && j % 2 == 0)
                    {
                        string tileType = "";
                        try
                        {
                            tileType = map.Tilesets[0].Tiles[tile].Type;
                        }
                        catch
                        {
                            tileType = "solid";
                        }

                        
                        
                    } 
                    */

                    

                    if (tile > 0)
                            tile--;
                    tileMap[i+j*64] = (byte)(tile);


                    if (collisions[tile] != 0)
                    {
                        //put 64 to objects map - 64 is solid
                        objectsMap[i + j * 64] = 64;
                    }
                    else
                    {
                        objectsMap[i + j * 64] = 255;
                    }
                }            
            }


        


            Console.WriteLine("Parsing OBJECTS!");
            int totalObjects = map.ObjectGroups["objects"].Objects.Count();
            Console.WriteLine("TOTAL OBJECTS: " + totalObjects.ToString());

            for (int i = 0; i < totalObjects; i++)
            {
                //string objectType = map.ObjectGroups["objects"].Objects[i].Type.ToString();                

                int x = (int)map.ObjectGroups["objects"].Objects[i].X;
                int y = (int)map.ObjectGroups["objects"].Objects[i].Y;
                int id = Int32.Parse(map.ObjectGroups["objects"].Objects[i].Properties["id"]);
                int tile = Int32.Parse(map.ObjectGroups["objects"].Objects[i].Properties["tile"]);

                gameObjects.Add(new GameObject((byte)(x/8),(byte)(y/8),(byte)(tile),(byte)(id)));

                // heroesList.Add(new Creature(heroesList.Count(), heroType, x, y, level));
            }


            Console.WriteLine("Parsing TRIGGERS!");
            int totalTriggers = map.ObjectGroups["triggers"].Objects.Count();
            Console.WriteLine("TOTAL TRIGGERS: " + totalTriggers.ToString());

            for (int i = 0; i < totalTriggers; i++)
            {
                //string objectType = map.ObjectGroups["objects"].Objects[i].Type.ToString();                

                int x = (int)map.ObjectGroups["triggers"].Objects[i].X;
                int y = (int)map.ObjectGroups["triggers"].Objects[i].Y;
                int id = Int32.Parse(map.ObjectGroups["triggers"].Objects[i].Properties["id"]);
                //put 64 to objects map - 64 is solid
                triggersMap[(byte)(x / 8) + (byte)(y / 8) * 64] = (byte)id;

                triggers[i * 4 + 0] = (byte)id;
                triggers[i * 4 + 1] = (byte)(x / 8);
                triggers[i * 4 + 2] = (byte)(y / 8);
                triggers[i * 4 + 3] = (byte)255;

                //int tile = Int32.Parse(map.ObjectGroups["objects"].Objects[i].Properties["tile"]);
                //int tile = 254;
                // gameObjects.Add(new GameObject((byte)(x / 8), (byte)(y / 8), (byte)(tile), (byte)(id)));

                // heroesList.Add(new Creature(heroesList.Count(), heroType, x, y, level));
            }


            using (var fs = new FileStream(args[0].Split('.')[0] + "-objects.bin", FileMode.Create, FileAccess.Write))
            {

                //generate binary file to creatures list
                foreach (GameObject c in gameObjects)
                {
                    fs.WriteByte((byte)(c.tile));
                    fs.WriteByte((byte)(c.id));
                    fs.WriteByte((byte)(c.x));
                    fs.WriteByte((byte)(c.y));                    
                }
                fs.WriteByte(255);
                fs.Close();
            }


            Console.WriteLine();

            //////////////////////////////////////////////////////////
            /*
            Console.WriteLine("Parsing Tiles Priorities!");
            //this property in tileset            
            int tilesInTileset = map.Tilesets[0].Tiles.Count();
            
            for (int i=0;i<256;i++)
            {
                try
                {
                    double prob = map.Tilesets[0].Tiles[i].Probability;
                    if (prob == 0)                    
                        priorities[i] = 1;
                }
                catch
                {
                    //no such field
                }
               
            }

            */




            File.WriteAllBytes(args[0].Split('.')[0] + ".bin", tileMap);
            File.WriteAllBytes(args[0].Split('.')[0] + ".obj", objectsMap);
            Array.Copy(triggers, 0,triggersMap,4096,256);
            File.WriteAllBytes(args[0].Split('.')[0] + ".trg", triggersMap);
            // File.WriteAllBytes(args[0].Split('.')[0] + ".trglist", triggers);

            File.WriteAllBytes("tilesPriorities.bin", priorities);
            File.WriteAllBytes("tilesCollisions.bin", collisions);
            File.WriteAllBytes("shadowMap.bin", shadowMap);



        }

            

                //===================================================

//                Console.ReadKey();
            
        }
          
        
}
